from types import GeneratorType
from unittest import TestCase, main

from incase import Case, case_modifier, incase, planetary_defense_shield


class TestExtra(TestCase):
    def test_incase_str_str(self):
        self.assertEqual(incase("snake", "example text"), "example_text")

    def test_incase_str_list(self):
        self.assertEqual(
            incase("upper", ["three", "word", "list"]), ["THREE", "WORD", "LIST"]
        )

    def test_incase_str_tuple(self):
        self.assertEqual(
            incase("upper", ("three", "word", "tuple")), ("THREE", "WORD", "TUPLE")
        )

    def test_incase_str_generator(self):
        generator = (word for word in ["some", "list"])
        incased = incase("upper", generator)
        self.assertIsInstance(incased, GeneratorType)
        self.assertEqual(list(incased), ["SOME", "LIST"])

    def test_incase_str_nested(self):
        nested = {
            "first_key": ["some", "list"],
            "second_key": {"another": "dict"},
            "third_key": 1,
        }
        self.assertEqual(
            incase("upper", nested),
            {
                "first_key": ["SOME", "LIST"],
                "second_key": {"another": "DICT"},
                "third_key": 1,
            },
        )

    def test_incase_mapping_list(self):
        self.assertEqual(
            incase(["upper", "lower", "snake", "camel"], ["some_word"] * 4),
            ["SOME WORD", "some word", "some_word", "someWord"],
        )

    def test_incase_mapping_dict(self):
        self.assertEqual(
            incase(
                {"first_key": "alternating", "second_key": "title"},
                {"first_key": "some example", "second_key": "another example"},
            ),
            {"first_key": "SoMe eXaMpLe", "second_key": "Another Example"},
        )

    def test_incase_mapping_str(self):
        self.assertEqual(incase({"bob": "upper"}, "bob"), "BOB")

    def test_incase_case_none(self):
        self.assertIsNone(incase("upper", None))

    def test_incase_none_none(self):
        self.assertIsNone(incase(None, None))

    def test_incase_case_str(self):
        self.assertEqual(incase(Case.UPPER, "this"), "THIS")

    def test_incase_exception(self):
        with self.assertRaises(NotImplementedError):
            incase(1, "test")

    def test_case_modifier(self):
        @case_modifier(
            args_case="upper",
            keywords_case="snake",
            kwargs_case="upper",
            output_case="upper",
        )
        def test_func(arg1, *, kwarg_1, **kwargs):
            self.assertEqual(arg1, arg1.upper())
            self.assertEqual(kwarg_1, kwarg_1.upper())
            for kwarg_name in kwargs.keys():
                self.assertEqual(kwarg_name, kwarg_name.lower())
            return "test"

        result = test_func("something lower", kwarg_1="also lower", KWARG_2=None)
        self.assertEqual(result, "TEST")

    def test_planetary_defense(self):
        values_dict = {"thing": "stuff"}
        planetary_defense_shield("upper", values_dict)
        self.assertEqual(values_dict["thing"], values_dict["THING"])


if __name__ == "__main__":
    main()  # pragma: no cover



from unittest import TestCase, main

from incase.middleware import JSONCaseTranslatorMiddleware
from starlette.applications import Starlette
from starlette.responses import JSONResponse, PlainTextResponse
from starlette.routing import Route
from starlette.testclient import TestClient

CORS_HEADERS = {
    "access-control-allow-origin": "*",  # Allows any origin
    "access-control-allow-methods": "POST, GET, OPTIONS, PUT, DELETE",  # Allowed methods
    "access-control-allow-headers": "Content-Type, Access-Control-Allow-Headers, Authorization, X-Requested-With",  # Allowed headers
}


def plain_text_endpoint(request):
    return PlainTextResponse("hello world!")


def generic_get(request):
    if request.headers["camelCase"] != "yes":
        raise ValueError("Where did my header go?")
    return JSONResponse({"first_thing": 1, "second_thing": 2})


async def greeting(request):
    data = await request.json()
    first_name = data["first_name"]
    last_name = data["last_name"]
    message = {"hello": f"{first_name} {last_name}"}
    return JSONResponse(message, headers=CORS_HEADERS)


routes = [
    Route("/", plain_text_endpoint),
    Route("/json", generic_get),
    Route("/greeting", endpoint=greeting, methods=["POST"]),
]

app = Starlette(debug=True, routes=routes)

app.add_middleware(JSONCaseTranslatorMiddleware)

client = TestClient(app)


class TestCaseTranslateMiddleware(TestCase):
    def test_plain_text(self):
        response = client.get("/", headers={"camelCase": "yes"})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.text, "hello world!")

    def test_get(self):
        response = client.get("/json", headers={"camelCase": "yes"})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), {"firstThing": 1, "secondThing": 2})

    def test_post(self):
        response = client.post(
            "/greeting",
            json={"firstName": "Patrick", "lastName": "Shechet"},
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), {"hello": "Patrick Shechet"})
        for key, value in CORS_HEADERS.items():
            self.assertEqual(value, response.headers[key])


if __name__ == "__main__":
    main()



import argparse
import sys

from incase.core import Case, Caseless
from incase.extra import case_modifier, incase, keys_case, planetary_defense_shield


def parse_args(args):
    parser = argparse.ArgumentParser(
        description="Convert a word from one case to another."
    )
    parser.add_argument(
        "words", type=str, nargs="+", help="The words you wish to change the case of."
    )
    parser.add_argument(
        "--case",
        dest="case",
        action="store",
        default="snake",
        help="The desired case to transform to.",
    )

    return parser.parse_args(args)


def cli():
    args = parse_args(sys.argv[1:])
    for word in args.words:
        print(incase(args.case, word))


if __name__ == "__main__":  # pragma: no cover
    cli()



from unittest import TestCase, main

from incase import Case, Caseless


class TestCaseless(TestCase):

    test_string = "test string"

    expected = {
        "CASELESS": Caseless("test string"),
        "CAMEL": "testString",
        "DROMEDARY": "testString",
        "MEDIAL": "testString",
        "SNAKE": "test_string",
        "PASCAL": "TestString",
        "INITIAL_CAPITALS": "TestString",
        "KEBAB": "test-string",
        "DASH": "test-string",
        "UPPER_SNAKE": "TEST_STRING",
        "UPPERCASE": "TEST STRING",
        "UPPER": "TEST STRING",
        "LOWERCASE": "test string",
        "LOWER": "test string",
        "TITLE": "Test String",
        "ALTERNATING": "TeSt sTrIng",
        "SARCASM": "TeSt sTrIng",
        "ORIGINAL": "test string",
        "WORD": "test string",
        Case.CASELESS: Caseless("test string"),
        Case.CAMEL: "testString",
        Case.DROMEDARY: "testString",
        Case.MEDIAL: "testString",
        Case.SNAKE: "test_string",
        Case.PASCAL: "TestString",
        Case.INITIAL_CAPITALS: "TestString",
        Case.KEBAB: "test-string",
        Case.DASH: "test-string",
        Case.UPPER_SNAKE: "TEST_STRING",
        Case.UPPERCASE: "TEST STRING",
        Case.UPPER: "TEST STRING",
        Case.LOWERCASE: "test string",
        Case.LOWER: "test string",
        Case.TITLE: "Test String",
        Case.ALTERNATING: "TeSt sTrIng",
        Case.SARCASM: "TeSt sTrIng",
        Case.ORIGINAL: "test string",
        Case.WORD: "test string",
    }

    def test_cases(self):
        for case, value in self.expected.items():
            with self.subTest(msg=f"test {case}", case=case, value=value):
                self.assertEqual(Caseless(self.test_string)[case], value)

    def test_slice(self):
        self.assertEqual(self.test_string[1::2], Caseless(self.test_string)[1::2])

    def test_repr(self):
        self.assertEqual(repr(Caseless(self.test_string)), f'Caseless("{self.test_string}")')

    def test_hash(self):
        self.assertEqual(
            hash(Caseless(self.test_string)), hash(Caseless(self.test_string.upper()))
        )

    def test_factory(self):
        factory_func = Caseless.factory("upper")
        self.assertEqual(factory_func(self.test_string), self.test_string.upper())


if __name__ == "__main__":
    main()  # pragma: no cover



import functools
import textwrap
import typing as t
import warnings
import weakref

from incase.core import Case, Caseless

DO_NOT_TOUCH = [
    "__name__",
    "__doc__",
    "__package__",
    "__loader__",
    "__spec__",
    "__file__",
    "__cached__",
    "__builtins__",
]


def _incase_iterable(case: t.Iterable, value: t.Iterable) -> t.Iterable:
    return (Caseless(target)[case] for case, target in zip(case, value))


def _incase_single(case: str | Case, value: str | Case | dict | t.Iterable) -> str:
    if not value:
        return value
    if isinstance(value, (str, Case)):
        return Caseless(value)[case]
    elif isinstance(value, dict):
        return {k: _incase_single(case, v) for k, v in value.items()}
    else:
        try:
            if iter(value):
                generator = (_incase_single(case, v) for v in value)
                if isinstance(value, (list, tuple, set)):
                    return type(value)(generator)
                else:
                    return generator
        except TypeError:
            return value


@functools.singledispatch
def incase(case: t.Any, value: t.Any) -> t.Callable:
    if not value:
        return value
    # Enum is not a class and so single dispatch doesn't work on it...
    if isinstance(case, Case):
        return _incase_single(case.name, value)
    else:
        raise NotImplementedError(f"incase does not support type: {type(case)}")


@incase.register
def _(case: str, value: str | Case) -> dict:
    return _incase_single(case, value)


@incase.register
def _(case: list, incaseable: t.Iterable) -> t.List:
    return list(_incase_iterable(case, incaseable))


@incase.register
def _(case: dict, value: dict) -> dict:
    if isinstance(value, dict):
        return {k: Caseless(v)[case[k]] for k, v in value.items()}
    else:
        return incase(case[value], value) if value in case else value


def case_modifier(
    args_case: t.Sequence | str | Case | None = None,
    kwargs_case: dict | str | Case | None = None,
    keywords_case: str | Case | None = None,
    output_case: t.Sequence | str | Case | dict | None = None,
):
    def decorator(func):
        def wrapper(*args, **kwargs):
            args = incase(args_case, args) if args_case else args
            kwargs = incase(kwargs_case, kwargs) if kwargs_case else kwargs
            kwargs = (
                {incase(keywords_case, k): v for k, v in kwargs.items()}
                if keywords_case
                else kwargs
            )
            return incase(output_case, func(*args, **kwargs))

        return functools.wraps(func)(wrapper)

    return decorator


def planetary_defense_shield(case: str | Case, globals: dict):
    warnings.warn(
        textwrap.dedent(
            """
                                You have invoked the planetary defense shield.
                                This could have unknown consequences. 
                                At the least, it will increase the memory use of your code. 
                                It is not a good idea in production."""
        )
    )
    new_values = {}
    for k in globals.keys():
        if (
            k not in DO_NOT_TOUCH
            and k != (newname := incase(case, k))
            and newname not in globals
        ):
            print(f"cloaning {k} to {newname}")
            try:
                new_values[newname] = weakref.proxy(globals()[k])
            except TypeError:
                new_values[newname] = globals[k]

    globals.update(new_values)


def keys_case(obj: t.Any, case: Case) -> t.Any:
    if isinstance(obj, str):
        return obj

    try:
        return {Caseless(key)[case]: value for key, value in obj.items()}
    except AttributeError:
        try:
            return [keys_case(i, case) for i in obj]
        except TypeError:
            return obj



import argparse
from unittest import TestCase, main, mock

from incase import parse_args, cli


class TestCLI(TestCase):
    def test_argparser(self):
        args = parse_args(["some word", "--case", "upper"])
        self.assertEqual(args.words[0], "some word")
        self.assertEqual(args.case, "upper")

    @mock.patch("builtins.print")
    @mock.patch(
        "argparse.ArgumentParser.parse_args",
        return_value=argparse.Namespace(words=["this"], case="upper"),
    )
    def test_cli(self, mock_args, mock_print):
        cli()
        mock_print.assert_called_with("THIS")


if __name__ == "__main__":
    main()  # pragma: no cover



import json
import typing

from starlette.middleware.base import BaseHTTPMiddleware
from starlette.requests import Request
from starlette.responses import Response, JSONResponse

from incase import Case, Caseless, keys_case

RequestResponseEndpoint = typing.Callable[[Request], typing.Awaitable[Response]]


class camelJsonResponse(JSONResponse):
    def render(self, content: typing.Any) -> bytes:
        return json.dumps(keys_case(content, Case.CAMEL)).encode("utf-8")


async def make_camel_json(response: Response) -> Response:
    header = response.headers
    del header["content-length"]
    content = None
    async for chunk in response.body_iterator:
        if content is not None:  # Already have some content
            raise NotImplementedError("Streaming the response body isn't supported yet")
        if isinstance(chunk, bytes):
            content = chunk.decode("utf-8")
        else:
            content = chunk
    return camelJsonResponse(
        content=json.loads(content),
        status_code=response.status_code,
        headers=header,
        media_type=response.media_type,
        background=response.background,
    )


class JSONCaseTranslatorMiddleware(BaseHTTPMiddleware):
    """This middleware translates the case of json keys recieved and sent by the
    asgi app. It is helpful for allowing a python back-end to use snake_case
    while allowing a javascript front end to use camelCase."""

    def __init__(self, app, handle_response: bool = True):
        super().__init__(app)
        self.handle_response = handle_response

    async def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
        try:
            body = await request.body()
            data = json.loads(body)
            request._body = json.dumps(keys_case(data, Case.SNAKE)).encode(
                encoding="utf-8"
            )
            request.content_length = len(request._body)
        except json.JSONDecodeError:
            pass  # guess it wasn't json

        response = await call_next(request)
        if (
            self.handle_response
            and response.headers.get("content-type") == "application/json"
        ):
            new_response = await make_camel_json(response)
            return new_response
        else:
            return response



import enum
import re
import typing as t
from collections import UserString


WHITESPACE = re.compile(r"\s|-")
UPPER_DELIMITED_TO_SNAKE_1 = re.compile(r"([A-Z]+)([A-Z][a-z])")
UPPER_DELIMITED_TO_SNAKE_2 = re.compile(r"([a-z\d])([A-Z])")
UPPER_DELIMITED_TO_SNAKE_REPL = r"\1_\2"


class Case(enum.Enum):
    CASELESS = 0
    CAMEL = 1
    DROMEDARY = 1
    MEDIAL = 1
    SNAKE = 2
    PASCAL = 3
    INITIAL_CAPITALS = 3
    KEBAB = 4
    DASH = 4
    UPPER_SNAKE = 5
    UPPERCASE = 6
    UPPER = 6
    LOWERCASE = 7
    LOWER = 7
    TITLE = 8
    ALTERNATING = 9
    SARCASM = 9
    ORIGINAL = 10
    WORD = 11


class Caseless(str, UserString):
    def snakify(self, string: str) -> str:
        # Trim, Replace whitespace and - with _
        string = WHITESPACE.sub("_", string.strip())
        # Camel/Pascal to snake and lowercase
        return UPPER_DELIMITED_TO_SNAKE_2.sub(
            UPPER_DELIMITED_TO_SNAKE_REPL,
            UPPER_DELIMITED_TO_SNAKE_1.sub(UPPER_DELIMITED_TO_SNAKE_REPL, string),
        ).lower()

    @property
    def parts(self):
        return self.snakify(self.data).split("_")

    @property
    def snake(self) -> str:
        return "_".join(self.parts)

    @property
    def upper_snake(self) -> str:
        return self.snake.upper()

    @property
    def kebab(self) -> str:
        return self.snake.replace("_", "-")

    @property
    def upper(self) -> str:
        return self.word.upper()

    @property
    def lower(self) -> str:
        return self.word.lower()

    @property
    def title(self) -> str:
        return self.word.title()

    @property
    def camel(self) -> str:
        return "".join([self.parts[0], *[word.title() for word in self.parts[1:]]])

    @property
    def pascal(self) -> str:
        return "".join([word.title() for word in self.parts])

    @property
    def alternating(self):
        uppercase = [word.upper() for word in self.word[0::2]]
        lowercase = [word.lower() for word in self.word[1::2]]
        return "".join([val for pair in zip(uppercase, lowercase) for val in pair]) + (
            self.word[-1] if len(self.word) % 2 == 1 else ""
        )

    @property
    def original(self):
        return self.data

    @property
    def word(self) -> str:
        return " ".join(self.parts)

    def __getitem__(self, key: int | slice | str | Case) -> str:
        if isinstance(key, (int, slice)):
            return self.data[key]

        if isinstance(key, str):
            key = Case[key.upper()]

        match key:
            case Case.CASELESS:
                return self
            case Case.CAMEL:
                return self.camel
            case Case.SNAKE:
                return self.snake
            case Case.PASCAL:
                return self.pascal
            case Case.KEBAB:
                return self.kebab
            case Case.UPPER_SNAKE:
                return self.upper_snake
            case Case.UPPERCASE:
                return self.upper
            case Case.LOWERCASE:
                return self.lower
            case Case.TITLE:
                return self.title
            case Case.ALTERNATING:
                return self.alternating
            case Case.ORIGINAL:
                return self.original
            case Case.WORD:
                return self.word

    def __eq__(self, other):
        return self.parts == Caseless(other).parts

    def __repr__(self) -> str:
        return f'Caseless("{self.data}")'

    def __hash__(self) -> str:
        return hash(tuple(self.parts))

    @classmethod
    def factory(cls, case: str | Case) -> t.Callable[[str], str]:
        def factory_func(word: str) -> str:
            return cls(word)[case]

        return factory_func
